@@ -337,19 +337,19 @@ gen_vec <- function(data, p = 2, exogen = NULL, s = 2, r = NULL,
   }
   model[["rank"]] = 0
   
-  if (class(structural) == "logical") {
+  if ("logical" %in% class(structural)) {
     model[["structural"]] <- structural
   } else {
     stop("Argument 'structural' must be of class 'logical'.")
   }
   
   # TVP specifications ----
-  if (class(tvp) == "logical") {
+  if ("logical" %in% class(tvp)) {
     model[["tvp"]] <- tvp 
   } else {
     stop("Argument 'tvp' must be of class 'logical'.")
   }
-  if (class(sv) == "logical") {
+  if ("logical" %in% class(sv)) {
     model[["sv"]] <- sv
   } else {
     stop("Argument 'sv' must be of class 'logical'.")
