@@ -328,7 +328,7 @@ Rcpp::List bvectvpalg(Rcpp::List object) {
     }
   }
   // Initial values
-  arma::vec h_init, sigma_h, u_vec, sigma_post_scale;
+  arma::vec h_constant, h_init, sigma_h, u_vec, sigma_post_scale;
   arma::mat h_init_post_v, sigma_h_i, diag_sigma_i_temp;
   arma::vec h_init_post_mu;
   arma::mat h, h_lag, sse;
@@ -339,6 +339,7 @@ Rcpp::List bvectvpalg(Rcpp::List object) {
     h = Rcpp::as<arma::mat>(init_sigma["h"]);
     h_lag = h * 0;
     sigma_h = Rcpp::as<arma::vec>(init_sigma["sigma_h"]);
+    h_constant = Rcpp::as<arma::vec>(init_sigma["constant"]);
     h_init = arma::vectorise(h.row(0));
     sigma_u_i = arma::diagmat(1 / exp(h_init));
   } else {
@@ -624,18 +625,12 @@ Rcpp::List bvectvpalg(Rcpp::List object) {
       u = arma::reshape(Psi * arma::vectorise(u), k, tt);
     }
     
+    ////////////////////////////////////////////////////////////////////////////
+    // Draw measurement error variances
+    
     if (sv) {
       
-      // Draw variances
-      for (int i = 0; i < k; i++) {
-        h.col(i) = bvartools::stoch_vol(u.row(i).t(), h.col(i), sigma_h(i), h_init(i), .0001);
-      }
-      diag_omega_i.diag() = 1 / exp(arma::vectorise(h.t()));
-      if (covar) {
-        diag_sigma_u_i = arma::trans(Psi) * diag_omega_i * Psi;
-      } else {
-        diag_sigma_u_i = diag_omega_i;
-      }
+      h = bvartools::stochvol_ksc1998(arma::trans(u), h, sigma_h, h_init, h_constant);
       
       // Draw sigma_h
       h_lag.row(0) = h_init.t();
@@ -655,19 +650,32 @@ Rcpp::List bvectvpalg(Rcpp::List object) {
     } else {
       
       if (use_gamma) {
-        
         sse = u * u.t();
         for (int i = 0; i < k; i++) {
           omega_i(i, i) = arma::randg<double>(arma::distr_param(sigma_post_shape(i), 1 / arma::as_scalar(sigma_prior_rate(i) + sse(i, i) * 0.5)));
         }
-        diag_omega_i = arma::kron(diag_tt, omega_i);
+      }
+      
+    }
+    
+    ////////////////////////////////////////////////////////////////////////////
+    // Combine Psi and Omega resp. draw from Wishart
+    
+    if (sv) {
+      diag_omega_i.diag() = 1 / exp(arma::vectorise(h.t()));
+      if (covar) {
+        diag_sigma_u_i = arma::trans(Psi) * diag_omega_i * Psi;
+      } else {
+        diag_sigma_u_i = diag_omega_i;
+      }
+    } else {
+      if (use_gamma) {
         if (covar) {
           diag_sigma_u_i = arma::trans(Psi) * diag_omega_i * Psi;
         } else {
           sigma_u_i = omega_i;
           diag_sigma_u_i = diag_omega_i;
         }
-        
       } else {
         sigma_u_i = arma::wishrnd(arma::solve(sigma_prior_scale + u * u.t(), diag_k), sigma_post_df);
         diag_sigma_u_i = arma::kron(diag_tt, sigma_u_i);
